@@ -1,5 +1,5 @@
 ### Terrence D. Jorgensen & Sunthud Pornprasertmanit
-### Last updated: 3 July 2021
+### Last updated: 9 May 2022
 ### source code for compareFit() function and FitDiff class
 
 
@@ -94,10 +94,12 @@ setMethod("show", signature(object = "FitDiff"), function(object) {
 ##'   "rmsea", "srmr", "aic", "bic")}. If \code{"all"}, all available fit
 ##'   measures will be returned.
 ##' @param nd number of digits printed
+##' @param tag single \code{character} used to flag the model preferred by each
+##'   fit index. To omit tags, set to \code{NULL} or \code{NA}.
 ##'
 ##' @export
 setMethod("summary", signature(object = "FitDiff"),
-          function(object, fit.measures = "default", nd = 3) {
+          function(object, fit.measures = "default", nd = 3, tag = "\u2020") {
 
   if (nrow(object@nested) > 0L) {
 		cat("################### Nested Model Comparison #########################\n")
@@ -148,7 +150,8 @@ setMethod("summary", signature(object = "FitDiff"),
     minvalue[!badness & !goodness] <- NA
     fit.integer <- grepl(pattern = c("df|npar|ntotal"),
                          x = colnames(fit.indices))
-    suppressWarnings(fitTab <- as.data.frame(mapply(tagDagger, nd = nd,
+    suppressWarnings(fitTab <- as.data.frame(mapply(tagCharacter, nd = nd,
+                                                    char = tag,
                                                     vec = fit.indices,
                                                     minvalue = minvalue,
                                                     print_integer = fit.integer),
@@ -287,7 +290,7 @@ saveFileFitDiff <- function(object, file, what = "summary",
 ##'                         moreIndices = TRUE) # include moreFitIndices()
 ##' summary(measEqOut)
 ##' summary(measEqOut, fit.measures = "all")
-##' summary(measEqOut, fit.measures = c("aic", "bic", "sic"))
+##' summary(measEqOut, fit.measures = c("aic", "bic", "sic", "ibic"))
 ##'
 ##'
 ##' \dontrun{
@@ -439,7 +442,7 @@ compareFit <- function(..., nested = TRUE, argsLRT = list(), indices = TRUE,
 	## TEST STATISTICS
 	if (nested) {
 
-	  if (class(mods[[1]]) == "lavaan") {
+	  if (inherits(mods[[1]], "lavaan")) {
 	    argsLRT$model.names <- names(mods)
 	    argsLRT$object <- mods[[1]]
 	    nestedout <- do.call(lavTestLRT, c(mods[-1], argsLRT))
@@ -506,7 +509,14 @@ noLeadingZero <- function(vec, fmt, nd = 3L) {
   out
 }
 
-tagDagger <- function(vec, minvalue = NA, print_integer = FALSE, nd = 3L) {
+tagCharacter <- function(vec, char = "\u2020", minvalue = NA,
+                         print_integer = FALSE, nd = 3L) {
+  char <- if (is.null(char)) as.character(NA) else as.character(char)[1]
+  if (nchar(char) != 1L) {
+    message('Only a single character can be used to tag= preferred models, so ',
+            'no tags were added.  To omit tags, specify tag=NULL or tag=NA.')
+    char <- as.character(NA)
+  }
   if (print_integer) {
     vec <- noLeadingZero(vec, fmt = "%.0f", nd = nd)
   } else if (is.na(minvalue)) {
@@ -514,7 +524,7 @@ tagDagger <- function(vec, minvalue = NA, print_integer = FALSE, nd = 3L) {
   } else {
     target <- if (minvalue) min(vec, na.rm = TRUE) else max(vec, na.rm = TRUE)
     tag <- rep(" ", length(vec))
-    tag[vec == target] <- "\u2020"
+    if (!is.na(char)) tag[vec == target] <- char
     vec <- noLeadingZero(vec, fmt = paste0("%.", nd, "f"), nd = nd)
     vec <- paste0(vec, tag)
   }
