@@ -1,9 +1,7 @@
 ### Title: Compute more fit indices
-### Authors: Terrence D. Jorgensen <TJorgensen314@gmail.com>
-###          Sunthud Pornprasertmanit <psunthud@ku.edu>,
-###          Aaron Boulton <aboulton@ku.edu>,
-###          Ruben Arslan <rubenarslan@gmail.com>
-### Last updated: 3 July 2021
+### Authors: Terrence D. Jorgensen, Sunthud Pornprasertmanit,
+###          Aaron Boulton, Ruben Arslan
+### Last updated: 18 September 2021
 ### Description: Calculations for promising alternative fit indices
 
 
@@ -12,10 +10,14 @@
 ##'
 ##' Calculate more fit indices that are not already provided in lavaan.
 ##'
-##' Gamma Hat (gammaHat; West, Taylor, & Wu, 2012) is a global fit index which
-##' can be computed (assuming equal number of indicators across groups) by
+##' See \code{\link{nullRMSEA}} for the further details of the computation of
+##' RMSEA of the null model.
 ##'
-##' \deqn{ gammaHat =\frac{p}{p + 2 \times \frac{\chi^{2}_{k} - df_{k}}{N}} ,}
+##' Gamma-Hat (\code{gammaHat}; West, Taylor, & Wu, 2012) is a global
+##' goodness-of-fit index which can be computed (assuming equal number of
+##' indicators across groups) by
+##'
+##'   \deqn{ \hat{\Gamma} =\frac{p}{p + 2 \times \frac{\chi^{2}_{k} - df_{k}}{N}},}
 ##'
 ##' where \eqn{p} is the number of variables in the model, \eqn{\chi^{2}_{k}} is
 ##' the \eqn{\chi^2} test statistic value of the target model, \eqn{df_{k}} is
@@ -23,89 +25,125 @@
 ##' sample size (or sample size minus the number of groups if \code{mimic} is
 ##' set to \code{"EQS"}).
 ##'
-##' Adjusted Gamma Hat (adjGammaHat; West, Taylor, & Wu, 2012) is a global fit
-##' index which can be computed by
+##' Adjusted Gamma-Hat (\code{adjGammaHat}; West, Taylor, & Wu, 2012) is a
+##' global fit index which can be computed by
 ##'
-##' \deqn{ adjGammaHat = \left(1 - \frac{K \times p \times (p + 1)}{2 \times
-##' df_{k}} \right) \times \left( 1 - gammaHat \right) ,}
+##'   \deqn{ \hat{\Gamma}_\textrm{adj} = \left(1 - \frac{K \times p \times
+##'   (p + 1)}{2 \times df_{k}} \right) \times \left( 1 - \hat{\Gamma} \right),}
 ##'
-##' where \eqn{K} is the number of groups (please refer to Dudgeon, 2004 for the
-##' multiple-group adjustment for agfi*).
+##' where \eqn{K} is the number of groups (please refer to Dudgeon, 2004, for
+##' the multiple-group adjustment for \code{adjGammaHat}).
 ##'
-##' Corrected Akaike Information Criterion (aic.smallN; Burnham & Anderson,
-##' 2003) is a corrected version of AIC for small sample size, often abbreviated
-##' AICc:
+##' The remaining indices are information criteria calculated using the
+##' \code{object}'s \eqn{-2 \times} log-likelihood, abbreviated \eqn{-2LL}.
 ##'
-##' \deqn{ aic.smallN = AIC + \frac{2k(k + 1)}{N - k - 1},}
+##' Corrected Akaike Information Criterion (\code{aic.smallN}; Burnham &
+##' Anderson, 2003) is a corrected version of AIC for small sample size, often
+##' abbreviated AICc:
 ##'
-##' where \eqn{AIC} is the original AIC: \eqn{-2 \times LL + 2k} (where \eqn{k}
+##'   \deqn{ \textrm{AIC}_{\textrm{small}-N} = AIC + \frac{2q(q + 1)}{N - q - 1},}
+##'
+##' where \eqn{AIC} is the original AIC: \eqn{-2LL + 2q} (where \eqn{q}
 ##' = the number of estimated parameters in the target model). Note that AICc is
 ##' a small-sample correction derived for univariate regression models, so it is
 ##' probably \emph{not} appropriate for comparing SEMs.
 ##'
-##' Corrected Bayesian Information Criterion (bic.priorN; Kuha, 2004) is similar
-##' to BIC but explicitly specifying the sample size on which the prior is based
-##' (\eqn{N_{prior}}).
+##' Corrected Bayesian Information Criterion (\code{bic.priorN}; Kuha, 2004) is
+##' similar to BIC but explicitly specifying the sample size on which the prior
+##' is based (\eqn{N_{prior}}) using the \code{nPrior} argument.
 ##'
-##' \deqn{ bic.priorN = f + k\log{(1 + N/N_{prior})},}
+##'   \deqn{ \textrm{BIC}_{\textrm{prior}-N} = -2LL + q\log{( 1 + \frac{N}{N_{prior}} )}.}
 ##'
-##' Stochastic information criterion (SIC; Preacher, 2006) is similar to AIC or
-##' BIC. This index will account for model complexity in the model's function
-##' form, in addition to the number of free parameters. This index will be
-##' provided only when the \eqn{\chi^2} value is not scaled. The SIC can be
+##' Bollen et al. (2014) discussed additional BICs that incorporate more terms
+##' from a Taylor series expansion, which the standard BIC drops.  The "Scaled
+##' Unit-Information Prior" BIC is calculated depending on whether the product
+##' of the vector of estimated model parameters (\eqn{\hat{\theta}}) and the
+##' observed information matrix (FIM) exceeds the number of estimated model
+##' parameters (Case 1) or not (Case 2), which is checked internally:
+##'
+##'   \deqn{ \textrm{SPBIC}_{\textrm{Case 1}} = -2LL + q(1 - \frac{q}{\hat{\theta}^{'} \textrm{FIM} \hat{\theta}}), or}
+##'   \deqn{ \textrm{SPBIC}_{\textrm{Case 2}} = -2LL + \hat{\theta}^{'} \textrm{FIM} \hat{\theta},}
+##'
+##' Bollen et al. (2014) credit the HBIC to Haughton (1988):
+##'
+##'   \deqn{ \textrm{HBIC}_{\textrm{Case 1}} = -2LL - q\log{2 \times \pi},}
+##'
+##' and proposes the information-matrix-based BIC by adding another term:
+##'
+##'   \deqn{ \textrm{IBIC}_{\textrm{Case 1}} = -2LL - q\log{2 \times \pi} - \log{\det{\textrm{ACOV}}},}
+##'
+##' Stochastic information criterion (SIC; see Preacher, 2006, for details) is
+##' similar to IBIC but does not subtract the term \eqn{q\log{2 \times \pi}}
+##' that is also in HBIC. SIC and IBIC account for model complexity in a model's
+##' functional form, not merely the number of free parameters.  The SIC can be
 ##' computed by
 ##'
-##' \deqn{ sic = \frac{1}{2}\left(f - \log{\det{I(\hat{\theta})}}\right),}
+##'   \deqn{ \textrm{SIC} = -2LL + \log{\det{\textrm{FIM}^{-1}}} = -2LL - \log{\det{\textrm{ACOV}}},}
 ##'
-##' where \eqn{I(\hat{\theta})} is the information matrix of the parameters.
+##' where the inverse of FIM is the asymptotic sampling covariance matrix (ACOV).
 ##'
-##' Hannan-Quinn Information Criterion (hqc; Hannan & Quinn, 1979) is used for
-##' model selection similar to AIC or BIC.
+##' Hannan--Quinn Information Criterion (HQC; Hannan & Quinn, 1979) is used for
+##' model selection, similar to AIC or BIC.
 ##'
-##' \deqn{ hqc = f + 2k\log{(\log{N})},}
+##' \deqn{ \textrm{HQC} = -2LL + 2k\log{(\log{N})},}
 ##'
-##' Note that if Satorra--Bentler or Yuan--Bentler's method is used, the fit
+##' Note that if Satorra--Bentler's or Yuan--Bentler's method is used, the fit
 ##' indices using the scaled \eqn{\chi^2} values are also provided.
 ##'
-##' See \code{\link{nullRMSEA}} for the further details of the computation of
-##' RMSEA of the null model.
-##'
 ##'
 ##' @importFrom lavaan lavInspect
 ##'
 ##' @param object The lavaan model object provided after running the \code{cfa},
-##' \code{sem}, \code{growth}, or \code{lavaan} functions.
+##'   \code{sem}, \code{growth}, or \code{lavaan} functions.
 ##' @param fit.measures Additional fit measures to be calculated. All additional
-##' fit measures are calculated by default
+##'   fit measures are calculated by default
 ##' @param nPrior The sample size on which prior is based. This argument is used
-##' to compute BIC*.
-##' @return \enumerate{
-##'  \item \code{gammaHat}: Gamma Hat
-##'  \item \code{adjGammaHat}: Adjusted Gamma Hat
-##'  \item \code{baseline.rmsea}: RMSEA of the Baseline (Null) Model
-##'  \item \code{aic.smallN}: Corrected (for small sample size) Akaike Information Criterion
-##'  \item \code{bic.priorN}: Bayesian Information Criterion with specified prior sample size
-##'  \item \code{sic}: Stochastic Information Criterion
-##'  \item \code{hqc}: Hannan-Quinn Information Criterion
-##'  \item \code{gammaHat.scaled}: Gamma Hat using scaled \eqn{\chi^2}
-##'  \item \code{adjGammaHat.scaled}: Adjusted Gamma Hat using scaled \eqn{\chi^2}
-##'  \item \code{baseline.rmsea.scaled}: RMSEA of the Baseline (Null) Model using scaled \eqn{\chi^2}
+##'   to compute \code{bic.priorN}.
+##'
+##' @return A \code{numeric} \code{lavaan.vector} including any of the
+##'   following requested via \code{fit.measures=}
+##' \enumerate{
+##'  \item \code{gammaHat}: Gamma-Hat
+##'  \item \code{adjGammaHat}: Adjusted Gamma-Hat
+##'  \item \code{baseline.rmsea}: RMSEA of the default baseline (i.e., independence) model
+##'  \item \code{gammaHat.scaled}: Gamma-Hat using scaled \eqn{\chi^2}
+##'  \item \code{adjGammaHat.scaled}: Adjusted Gamma-Hat using scaled \eqn{\chi^2}
+##'  \item \code{baseline.rmsea.scaled}: RMSEA of the default baseline (i.e.,
+##'        independence) model using scaled \eqn{\chi^2}
+##'  \item \code{aic.smallN}: Corrected (for small sample size) AIC
+##'  \item \code{bic.priorN}: BIC with specified prior sample size
+##'  \item \code{spbic}: Scaled Unit-Information Prior BIC (SPBIC)
+##'  \item \code{hbic}: Haughton's BIC (HBIC)
+##'  \item \code{ibic}: Information-matrix-based BIC (IBIC)
+##'  \item \code{sic}: Stochastic Information Criterion (SIC)
+##'  \item \code{hqc}: Hannan-Quinn Information Criterion (HQC)
 ##' }
+##'
 ##' @author Sunthud Pornprasertmanit (\email{psunthud@@gmail.com})
 ##'
 ##' Terrence D. Jorgensen (University of Amsterdam; \email{TJorgensen314@@gmail.com})
 ##'
-##' Aaron Boulton (University of North Carolina, Chapel Hill; \email{aboulton@@email.unc.edu})
+##' Aaron Boulton (University of Delaware)
 ##'
 ##' Ruben Arslan (Humboldt-University of Berlin, \email{rubenarslan@@gmail.com})
 ##'
 ##' Yves Rosseel (Ghent University; \email{Yves.Rosseel@@UGent.be})
 ##'
-##' @seealso \itemize{ \item \code{\link{miPowerFit}} For the modification
-##' indices and their power approach for model fit evaluation \item
-##' \code{\link{nullRMSEA}} For RMSEA of the null model }
+##' @seealso
+##' \itemize{
+##' \item \code{\link{miPowerFit}} For the modification indices and their
+##'        power approach for model fit evaluation
+##' \item \code{\link{nullRMSEA}} For RMSEA of the default independence model
+##' }
 ##'
-##' @references Burnham, K., & Anderson, D. (2003). \emph{Model selection and
+##' @references
+##'
+##' Bollen, K. A., Ray, S., Zavisca, J., & Harden, J. J. (2012). A comparison of
+##' Bayes factor approximation methods including two new methods.
+##' \emph{Sociological Methods & Research, 41}(2), 294--324.
+##' \doi{10.1177/0049124112452393}
+##'
+##' Burnham, K., & Anderson, D. (2003). \emph{Model selection and
 ##' multimodel inference: A practical--theoretic approach}. New York, NY:
 ##' Springer--Verlag.
 ##'
@@ -124,7 +162,8 @@
 ##'
 ##' West, S. G., Taylor, A. B., & Wu, W. (2012). Model fit and model selection
 ##' in structural equation modeling. In R. H. Hoyle (Ed.), \emph{Handbook of
-##' Structural Equation Modeling} (pp. 209--231). New York, NY: Guilford.
+##' structural equation modeling} (pp. 209--231). New York, NY: Guilford.
+##'
 ##' @examples
 ##'
 ##' HS.model <- ' visual  =~ x1 + x2 + x3
@@ -142,7 +181,7 @@ moreFitIndices <- function(object, fit.measures = "all", nPrior = 1) {
   ## check for validity of user-specified "fit.measures" argument
   fit.choices <- c("gammaHat","adjGammaHat","baseline.rmsea",
                    "gammaHat.scaled","adjGammaHat.scaled","baseline.rmsea.scaled",
-                   "aic.smallN","bic.priorN","hqc","sic")
+                   "aic.smallN","bic.priorN","spbic","hbic","ibic","sic","hqc")
   flags <- setdiff(fit.measures, c("all", fit.choices))
   if (length(flags)) stop(paste("Argument 'fit.measures' includes invalid options:",
                                 paste(flags, collapse = ", "),
@@ -163,10 +202,10 @@ moreFitIndices <- function(object, fit.measures = "all", nPrior = 1) {
   N <- n <- lavInspect(object, "ntotal")
   if (lavInspect(object, "options")$mimic == "EQS") n <- n - ngroup
 
-  # Calculate -2*log(likelihood)
+  ## Calculate -2*log(likelihood)
   f <- -2 * fit["logl"]
 
-  # Compute fit indices
+  ## Compute fit indices
   result <- list()
   if (length(grep("gamma", fit.measures, ignore.case = TRUE))) {
     gammaHat <- p / (p + 2 * ((fit["chisq"] - fit["df"]) / n))
@@ -186,6 +225,8 @@ moreFitIndices <- function(object, fit.measures = "all", nPrior = 1) {
       result["baseline.rmsea.scaled"] <- nullRMSEA(object, scaled = TRUE, silent = TRUE)
     }
   }
+
+  ## Compute information criteria
   if (!is.na(f)) {
     if ("aic.smallN" %in% fit.measures) {
       warning('AICc (aic.smallN) was developed for univariate linear models.',
@@ -195,11 +236,47 @@ moreFitIndices <- function(object, fit.measures = "all", nPrior = 1) {
     if ("bic.priorN" %in% fit.measures) {
       result["bic.priorN"] <- f + log(1 + N/nPrior) * nParam
     }
+    if ("spbic" %in% fit.measures) {
+      theta <- lavaan::coef(object)
+      FIM <- lavInspect(object, "information.observed")
+      junk <- t(theta) %*% FIM %*% theta
+      if (nParam < junk) {
+        result["spbic"] <- f + nParam*(1 - log(nParam / junk)) # Case 1
+      } else result["spbic"] <- f + junk # Case 2
+    }
+    if ("hbic" %in% fit.measures) result["hbic"] <- f - nParam*log(2*pi)
+
+    ## check determinant of ACOV for IBIC and SIC
+    if (any(c("ibic","sic") %in% fit.measures)) {
+      ACOV <- lavInspect(object, "vcov")
+      detACOV <- det(ACOV)
+      if (detACOV <= 0) {
+        ## look for duplicate names (simple equality constraints) to
+        ## remove any obviously redundant parameters
+        #TODO: any way to check for redundancies implied by "==" constraints?
+        RN <- unique(rownames(ACOV))
+        if (length(RN) < nrow(ACOV)) detACOV <- det(ACOV[RN, RN])
+      }
+    }
+    if ("ibic" %in% fit.measures) {
+      if (detACOV <= 0) {
+        result["ibic"] <- NA
+        message('Determinant of vcov(object) <= 0, so IBIC cannot be calculated')
+      } else result["ibic"] <- f - nParam*log(2*pi) - log(detACOV)
+    }
+    if ("sic" %in% fit.measures) {
+      if (detACOV <= 0) {
+        result["sic"] <- NA
+        message('Determinant of vcov(object) <= 0, so SIC cannot be calculated')
+      } else result["sic"] <- f - log(detACOV)
+    }
+
     if ("hqc" %in% fit.measures) result["hqc"] <- f + 2 * log(log(N)) * nParam
-    if ("sic" %in% fit.measures) result["sic"] <- sic(f, object)
   }
+
+  result <- unlist(result[fit.measures])
   class(result) <- c("lavaan.vector","numeric")
-  unlist(result[fit.measures])
+  result
 }
 
 
@@ -311,7 +388,7 @@ sic <- function(f, lresults = NULL) {
   if (DET <= 0) return(NA)
 
   ## return SIC
-  f + log(DET)
+
 }
 
 
