@@ -1,18 +1,19 @@
-### Ylenio Longo
-### Last updated: 10 January 2021
+### Ylenio Longo & Terrence D. Jorgensen
+### Last updated: 1 September 2021
 
-##' Assessing Discriminant Validity using Heterotrait-Monotrait Ratio
+##' Assessing Discriminant Validity using Heterotrait--Monotrait Ratio
 ##'
 ##' This function assesses discriminant validity through the
 ##' heterotrait-monotrait ratio (HTMT) of the correlations (Henseler, Ringlet &
-##' Sarstedt, 2015). Specifically, it assesses the geometric-mean correlation
-##' among indicators across constructs (i.e. heterotrait-heteromethod
+##' Sarstedt, 2015). Specifically, it assesses the arithmetic (Henseler et al.,
+##' ) or geometric (Roemer et al., 2021) mean correlation
+##' among indicators across constructs (i.e. heterotrait--heteromethod
 ##' correlations) relative to the geometric-mean correlation among indicators
-##' within the same construct (i.e. monotrait-heteromethod correlations).
-##' The resulting HTMT values are interpreted as estimates of inter-construct
+##' within the same construct (i.e. monotrait--heteromethod correlations).
+##' The resulting HTMT(2) values are interpreted as estimates of inter-construct
 ##' correlations. Absolute values of the correlations are recommended to
-##' calculate the HTMT matrix. Correlations are estimated using the
-##' \code{\link[lavaan]{lavCor}} function in the \pkg{lavaan} package.
+##' calculate the HTMT matrix, and are required to calculate HTMT2. Correlations
+##' are estimated using the \code{\link[lavaan]{lavCor}} function.
 ##'
 ##'
 ##' @importFrom stats cov2cor
@@ -33,11 +34,15 @@
 ##'   Treat these variables as ordered (ordinal) variables. Importantly, all
 ##'   other variables will be treated as numeric (unless \code{is.ordered} in
 ##'   \code{data}). (see also \link[lavaan]{lavCor})
-##' @param absolute logical. Whether HTMT values should be estimated based on
-##'   absolute correlations (recommended and default is \code{TRUE})
+##' @param absolute \code{logical} indicating whether HTMT values should be
+##'   estimated based on absolute correlations (default is \code{TRUE}). This
+##'   is recommended for HTMT but required for HTMT2 (so silently ignored).
+##' @param htmt2 \code{logical} indicating whether to use the geometric mean
+##'   (default, appropriate for congeneric indicators) or arithmetic mean
+##'   (which assumes tau-equivalence).
 ##'
-##' @return A matrix showing HTMT values (i.e., discriminant validity) between
-##'   each pair of factors
+##' @return A matrix showing HTMT(2) values (i.e., discriminant validity)
+##'   between each pair of factors.
 ##'
 ##' @author
 ##' Ylenio Longo (University of Nottingham; \email{yleniolongo@@gmail.com})
@@ -50,8 +55,13 @@
 ##'   modeling. \emph{Journal of the Academy of Marketing Science, 43}(1),
 ##'   115--135. \doi{10.1007/s11747-014-0403-8}
 ##'
+##'   Roemer, E., Schuberth, F., & Henseler, J. (2021). HTMT2-An improved
+##'   criterion for assessing discriminant validity in structural equation
+##'   modeling. \emph{Industrial Management & Data Systems}.
+##'   \doi{10.1108/IMDS-02-2021-0082}
+##'
 ##'   Voorhees, C. M., Brady, M. K., Calantone, R., & Ramirez, E. (2016).
-##'   Discriminant validity testing in marketing: an analysis, causes for
+##'   Discriminant validity testing in marketing: An analysis, causes for
 ##'   concern, and proposed remedies.
 ##'   \emph{Journal of the Academy of Marketing Science, 44}(1), 119--134.
 ##'   \doi{10.1007/s11747-015-0455-4}
@@ -65,9 +75,14 @@
 ##' dat <- HolzingerSwineford1939[, paste0("x", 1:9)]
 ##' htmt(HS.model, dat)
 ##'
+##' ## save covariance matrix
+##' HS.cov <- cov(HolzingerSwineford1939[, paste0("x", 1:9)])
+##' ## HTMT using arithmetic mean
+##' htmt(HS.model, sample.cov = HS.cov, htmt2 = FALSE)
+##'
 ##' @export
-htmt <- function (model, data = NULL, sample.cov = NULL, missing = "listwise",
-                  ordered = NULL, absolute = TRUE) {
+htmt <- function(model, data = NULL, sample.cov = NULL, missing = "listwise",
+                  ordered = NULL, absolute = TRUE, htmt2 = TRUE) {
   model <- lavaan::lavaanify(model)
   model <- model[model$op %in% "=~", ]
   factors <- unique(model$lhs)
@@ -100,13 +115,15 @@ htmt <- function (model, data = NULL, sample.cov = NULL, missing = "listwise",
       R <- sample.cov[varnames, varnames]
     }
   }
-  if (absolute) {
+  if (absolute || htmt2) {
     R <- abs(R)
   }
   diag(R) <- NA
   m.cor.w <- list()
   for (i in 1:nf) {
-    m.cor.w[[i]] <- mean(R[var[[i]], var[[i]]], na.rm = TRUE)
+    if (htmt2) {
+      m.cor.w[[i]] <- exp(mean(log(R[ var[[i]], var[[i]] ]), na.rm = TRUE))
+    } else m.cor.w[[i]] <- mean(R[ var[[i]], var[[i]] ], na.rm = TRUE)
   }
   m.cor.w <- as.numeric(m.cor.w)
   comb <- expand.grid(1:nf, 1:nf)
@@ -118,8 +135,12 @@ htmt <- function (model, data = NULL, sample.cov = NULL, missing = "listwise",
   paste(comb[, 2], comb[, 1])
   m.cor.a <- list()
   for (i in 1:nrow(comb)) {
-    m.cor.a[[i]] <- mean(R[var[[comb[i, 2]]],
-                           var[[comb[i, 1]]]], na.rm = TRUE)
+    if (htmt2) {
+      m.cor.a[[i]] <- exp(mean(log(R[ var[[comb[i, 2]]],
+                                      var[[comb[i, 1]]] ]),
+                               na.rm = TRUE))
+    } else m.cor.a[[i]] <- mean(R[ var[[ comb[i,2] ]],
+                                   var[[ comb[i,1] ]] ], na.rm = TRUE)
   }
   m.cor.a <- as.numeric(m.cor.a)
   outhtmt <- m.cor.a / g
