@@ -1,5 +1,5 @@
 ### Terrence D. Jorgensen & Yves Rosseel
-### Last updated: 29 August 2019
+### Last updated: 10 January 2021
 ### Pooled Wald test for multiple imputations
 ### Borrowed source code from lavaan/R/lav_test_Wald.R
 
@@ -61,7 +61,10 @@
 ##'   last option (\code{"no.npd"}) would exclude any imputations which
 ##'   yielded a nonpositive definite covariance matrix for observed or
 ##'   latent variables, which would include any "improper solutions" such
-##'   as Heywood cases.
+##'   as Heywood cases. Specific imputation numbers can also be included in this
+##'   argument, in case users want to  apply their own custom omission criteria
+##'   (or simulations can use different numbers of imputations without
+##'   redundantly refitting the model).
 ##' @param verbose \code{logical}. If \code{TRUE}, print the restriction
 ##'   matrix and the estimated restricted values.
 ##' @param warn \code{logical}. If \code{TRUE}, print warnings if they occur.
@@ -90,7 +93,7 @@
 ##'   Li, K.-H., Meng, X.-L., Raghunathan, T. E., & Rubin, D. B. (1991).
 ##'   Significance levels from repeated \emph{p}-values with multiply-imputed
 ##'   data. \emph{Statistica Sinica, 1}(1), 65--92. Retrieved from
-##'   https://www.jstor.org/stable/24303994
+##'   \url{https://www.jstor.org/stable/24303994}
 ##'
 ##'   Rubin, D. B. (1987). \emph{Multiple imputation for nonresponse in surveys}.
 ##'   New York, NY: Wiley.
@@ -155,6 +158,10 @@ lavTestWald.mi <- function(object, constraints = NULL, test = c("D1","D2"),
     Heywood.ov <- sapply(object@convergence, "[[", i = "Heywood.ov")
     useImps <- useImps & !(Heywood.lv | Heywood.ov)
   }
+  ## custom removal by imputation number
+  rm.imps <- omit.imps[ which(omit.imps %in% 1:length(useImps)) ]
+  if (length(rm.imps)) useImps[as.numeric(rm.imps)] <- FALSE
+  ## whatever is left
   m <- sum(useImps)
   if (m == 0L) stop('No imputations meet "omit.imps" criteria.')
   useImps <- which(useImps)
